<?php
/**
 * Custom 404 Page ACP Module
 *
 * @package mundophpbb\custom404\acp
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 */

namespace mundophpbb\custom404\acp;

class main_module
{
    public $u_action;
    public $tpl_name = '@mundophpbb_custom404/acp_custom404';
    public $page_title = 'ACP_CUSTOM404_TITLE';

    /** @var \phpbb\user */
    protected $user;

    /**
     * Main ACP module
     *
     * @param int    $id   The module ID
     * @param string $mode The mode (settings)
     */
    public function main($id, $mode)
    {
        global $phpbb_container, $user;

        // Services
        $this->user = $phpbb_container->get('user');
        $language = $phpbb_container->get('language');
        $request = $phpbb_container->get('request');
        $config = $phpbb_container->get('config');
        $template = $phpbb_container->get('template');
        $filesystem = $phpbb_container->get('filesystem');
        $upload_handler = $phpbb_container->get('mundophpbb.custom404.upload');

        // Load language files
		$user->add_lang_ext('mundophpbb/custom404', 'acp_common');
        //$this->load_language_files($language, $filesystem, $phpbb_container);
        add_form_key('mundophpbb_custom404');

        // Upload configuration
        $upload_dir = $phpbb_container->getParameter('core.root_path') . 'ext/mundophpbb/custom404/styles/all/theme/images/';
        $allowed_extensions = ['png', 'jpg', 'jpeg', 'gif'];
        $max_file_size = 2 * 1024 * 1024; // 2MB

        $success_message = '';
        $error_message = '';

        // Handle form submission for image upload
        if ($request->is_set_post('upload'))
        {
            if (!check_form_key('mundophpbb_custom404'))
            {
                $error_message = $language->lang('FORM_INVALID');
            }
            else
            {
                try
                {
                    $uploaded_file = $request->file('custom404_image_upload');
                    // Validate file type before calling upload handler
                    $ext = strtolower(pathinfo($uploaded_file['name'], PATHINFO_EXTENSION));
                    if (!in_array($ext, $allowed_extensions))
                    {
                        throw new \Exception($language->lang('ACP_CUSTOM404_INVALID_FILE_TYPE', implode(', ', array_map('strtoupper', $allowed_extensions))));
                    }
                    $new_image_path = $upload_handler->handle_upload($uploaded_file, $upload_dir, $allowed_extensions, $max_file_size);
                    if ($new_image_path)
                    {
                        $relative_image_path = 'ext/mundophpbb/custom404/styles/all/theme/images/' . basename($new_image_path);
                        $config->set('custom404_image_path', $relative_image_path);
                        $success_message = $language->lang('ACP_CUSTOM404_IMAGE_UPLOADED');
                    }
                }
                catch (\Exception $e)
                {
                    $error_message = $e->getMessage();
                }
            }
        }

        // Handle form submission for settings
        if ($request->is_set_post('submit'))
        {
            if (!check_form_key('mundophpbb_custom404'))
            {
                $error_message = $language->lang('FORM_INVALID');
            }
            else
            {
                $config->set('custom404_title', utf8_normalize_nfc($request->variable('custom404_title', $language->lang('CUSTOM404_DEFAULT_TITLE'), true)));
                $config->set('custom404_message', utf8_normalize_nfc($request->variable('custom404_message', $language->lang('CUSTOM404_DEFAULT_MESSAGE'), true)));
                $config->set('custom404_image_alt', utf8_normalize_nfc($request->variable('custom404_image_alt', $language->lang('CUSTOM404_DEFAULT_IMAGE_ALT'), true)));
                $config->set('custom404_image_path', $request->variable('custom404_image_path', ''));

                $success_message = $language->lang('ACP_CUSTOM404_SAVED');
            }
        }

        // Prepare BBCode message for display
        $message = $config['custom404_message'] ?? $language->lang('CUSTOM404_DEFAULT_MESSAGE');
        $uid = $bitfield = $flags = '';
        generate_text_for_storage($message, $uid, $bitfield, $flags, true, true, true);

        // Check SCEditor availability
        $core_path = $phpbb_container->getParameter('core.root_path');
        $sceditor_available = file_exists($core_path . 'adm/style/posting_buttons.html') && file_exists($core_path . 'adm/style/posting_editor.html');

        // Get images from directory
        $images = [];
        if ($filesystem->exists($upload_dir))
        {
            $files = scandir($upload_dir);
            foreach ($files as $file)
            {
                $ext = strtolower(pathinfo($file, PATHINFO_EXTENSION));
                if (in_array($ext, $allowed_extensions) && $filesystem->is_readable($upload_dir . $file))
                {
                    $images[] = $file;
                }
            }
            sort($images);
        }

        // Prepare image options with translated labels
        $image_options = [];
        $default_image = '404_error.png';
        $default_relative_path = 'ext/mundophpbb/custom404/styles/all/theme/images/' . $default_image;
        $default_absolute_path = generate_board_url() . '/' . $default_relative_path;

        $image_options[] = [
            'VALUE' => $default_relative_path,
            'SELECTED' => (empty($config['custom404_image_path']) || $config['custom404_image_path'] === $default_relative_path) ? ' selected="selected"' : '',
            'LABEL' => $language->lang('ACP_CUSTOM404_DEFAULT_IMAGE'),
            'ABSOLUTE_PATH' => $default_absolute_path
        ];

        foreach ($images as $image)
        {
            if ($image !== $default_image)
            {
                $encoded_image = rawurlencode($image);
                $relative_image_path = 'ext/mundophpbb/custom404/styles/all/theme/images/' . $encoded_image;
                $absolute_image_path = generate_board_url() . '/' . $relative_image_path;
                $image_options[] = [
                    'VALUE' => $relative_image_path,
                    'SELECTED' => ($config['custom404_image_path'] === $relative_image_path) ? ' selected="selected"' : '',
                    'LABEL' => htmlspecialchars_decode($image),
                    'ABSOLUTE_PATH' => $absolute_image_path
                ];
            }
        }

        // Set image path
        $selected_image_path = $config['custom404_image_path'] ? generate_board_url() . '/' . $config['custom404_image_path'] : $default_absolute_path;

        // Assign template variables
        $template->assign_vars([
            'U_ACTION' => $this->u_action,
            'CUSTOM404_TITLE' => $config['custom404_title'] ?? $language->lang('CUSTOM404_DEFAULT_TITLE'),
            'CUSTOM404_MESSAGE' => generate_text_for_edit($message, $uid, $flags)['text'],
            'CUSTOM404_IMAGE_ALT' => $config['custom404_image_alt'] ?? $language->lang('CUSTOM404_DEFAULT_IMAGE_ALT'),
            'CUSTOM404_IMAGE_PATH' => $selected_image_path,
            'DEFAULT_IMAGE_PATH' => $default_absolute_path,
            'S_BBCODE_ALLOWED' => true,
            'S_BBCODE_IMG' => true,
            'S_BBCODE_URL' => true,
            'S_BBCODE_QUOTE' => true,
            'S_BBCODE_FLASH' => false,
            'S_FORM_ENCODE' => ' enctype="multipart/form-data"',
            'S_BBCODE_EDITOR' => $sceditor_available,
            'SIG_EDIT' => false,
            'S_LINKS_ALLOWED' => true,
            'MAX_FONT_SIZE' => $config['max_post_font_size'] ?? 200,
            'ALLOWED_EXTENSIONS' => implode(', ', array_map('strtoupper', $allowed_extensions)),
            'MAX_FILE_SIZE_MB' => (int) ($max_file_size / 1024 / 1024),
            'UPLOAD_INFO' => $language->lang('ACP_CUSTOM404_UPLOAD_INFO', implode(', ', array_map('strtoupper', $allowed_extensions)), (int) ($max_file_size / 1024 / 1024)),
            'L_UPLOAD_IMAGE' => $language->lang('L_UPLOAD_IMAGE'),
            'L_SAVE_CHANGES' => $language->lang('L_SAVE_CHANGES'),
            'L_SELECT_IMAGE' => $language->lang('ACP_CUSTOM404_IMAGE_PATH_LABEL'),
            'L_CHOOSE_FILE' => $language->lang('ACP_CUSTOM404_CHOOSE_FILE'), // Nova string para "Escolher Arquivo"
            'L_NO_FILE_CHOSEN' => $language->lang('ACP_CUSTOM404_NO_FILE_CHOSEN'), // Nova string para "Nenhum arquivo escolhido"
            'S_SUCCESS_MESSAGE' => $success_message,
            'S_ERROR_MESSAGE' => $error_message,
        ]);

        // Assign image options
        foreach ($image_options as $option)
        {
            $template->assign_block_vars('image_options', $option);
        }

        // Include BBCode editor resources
        if ($sceditor_available)
        {
            $this->user->add_lang('posting');
            include_once($core_path . 'includes/functions_posting.' . $phpbb_container->getParameter('core.php_ext'));
        }
    }
}