<?php
/**
 * Custom 404 Page Event Listener
 *
 * @package mundophpbb\custom404\event
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 */
namespace mundophpbb\custom404\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Event listener for handling 404 errors
 */
class main_listener implements EventSubscriberInterface
{
    /** @var \phpbb\user */
    protected $user;

    /** @var \phpbb\language\language */
    protected $language;

    /** @var \phpbb\template\twig\twig */
    protected $template;

    /** @var \phpbb\config\config */
    protected $config;

    /** @var \phpbb\filesystem\filesystem */
    protected $filesystem;

    /** @var string */
    protected $root_path;

    /** @var bool */
    protected $is_apprequest = false;

    /**
     * Constructor
     *
     * @param \phpbb\user $user
     * @param \phpbb\language\language $language
     * @param \phpbb\template\twig\twig $template
     * @param \phpbb\config\config $config
     * @param \phpbb\filesystem\filesystem $filesystem
     * @param string $root_path
     */
    public function __construct(
        \phpbb\user $user,
        \phpbb\language\language $language,
        \phpbb\template\twig\twig $template,
        \phpbb\config\config $config,
        \phpbb\filesystem\filesystem $filesystem,
        $root_path
    )
    {
        $this->user = $user;
        $this->language = $language;
        $this->template = $template;
        $this->config = $config;
        $this->filesystem = $filesystem;
        $this->root_path = $root_path;
    }

    /**
     * Returns subscribed events
     *
     * @return array
     */
    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::REQUEST   => [['set_apprequest', 960]],
            KernelEvents::EXCEPTION => [['maybe_set_404', 960]],
        ];
    }

    /**
     * Sets the flag to use the custom 404 page for all styles
     *
     * @param GetResponseEvent $event
     */
    public function set_apprequest(GetResponseEvent $event)
    {
        $this->is_apprequest = true;
    }

    /**
     * Handles 404 exceptions and renders a custom 404 page
     *
     * @param GetResponseForExceptionEvent $event
     */
    public function maybe_set_404(GetResponseForExceptionEvent $event)
    {
        if ($this->is_apprequest && $event->getException() instanceof NotFoundHttpException)
        {
            // Load language files dynamically
            $this->load_language_files();

            // Log for debugging
            error_log('.: Handling 404 error with custom page at ' . date('Y-m-d H:i:s'));

            // Process BBCode for display
            $message = isset($this->config['custom404_message']) ? $this->config['custom404_message'] : $this->language->lang('CUSTOM404_DEFAULT_MESSAGE', 'CUSTOM404_DEFAULT_MESSAGE');
            $uid = $bitfield = $flags = '';
            generate_text_for_storage($message, $uid, $bitfield, $flags, true, true, true);
            $message = generate_text_for_display($message, $uid, $bitfield, $flags);

            // Validate image path
            $default_image_path = 'ext/mundophpbb/custom404/styles/all/theme/images/404_error.png';
            $image_path = isset($this->config['custom404_image_path']) && $this->config['custom404_image_path']
                ? $this->config['custom404_image_path']
                : $default_image_path;
            $full_image_path = $this->filesystem->realpath($this->root_path . $image_path);
            if (!$full_image_path || !$this->filesystem->exists($full_image_path))
            {
                $image_path = $default_image_path;
            }

            // Assign config values to template with language fallbacks
            $this->template->assign_vars([
                'CUSTOM404_TITLE'       => isset($this->config['custom404_title']) ? $this->config['custom404_title'] : $this->language->lang('CUSTOM404_DEFAULT_TITLE', 'CUSTOM404_DEFAULT_TITLE'),
                'CUSTOM404_MESSAGE'     => $message,
                'CUSTOM404_IMAGE_ALT'   => isset($this->config['custom404_image_alt']) ? $this->config['custom404_image_alt'] : $this->language->lang('CUSTOM404_DEFAULT_IMAGE_ALT', 'CUSTOM404_DEFAULT_IMAGE_ALT'),
                'CUSTOM404_IMAGE_PATH'  => $image_path,
            ]);

            // Set page header
            page_header(isset($this->config['custom404_title']) ? $this->config['custom404_title'] : $this->language->lang('CUSTOM404_DEFAULT_TITLE', 'CUSTOM404_DEFAULT_TITLE'), false);

            // Assign template file
            $this->template->set_filenames(['body' => '@mundophpbb_custom404/custom404.html']);

            // Set HTTP headers
            header('HTTP/1.1 404 Not Found', true, 404);
            header('Content-Type: text/html; charset=UTF-8');
            header('Cache-Control: no-cache, must-revalidate, max-age=0');

            // Render page
            page_footer(false);

            // Set the response to stop further processing
            $event->setResponse(new \Symfony\Component\HttpFoundation\Response(
                $this->template->assign_display('body'),
                404,
                ['Content-Type' => 'text/html; charset=UTF-8']
            ));
        }
    }

    /**
     * Load language files with fallback to English
     */
    protected function load_language_files()
    {
        $lang_path = $this->root_path . 'ext/mundophpbb/custom404/language/';
        $user_lang = $this->user->lang_name; // Use user->lang_name to get the user's selected language
        $fallback_lang = 'en'; // Default fallback language

        // Log user language for debugging
        error_log('.: User language: ' . $this->user->lang_name . ' at ' . date('Y-m-d H:i:s'));

        // Load primary language file
        $primary_lang_file = $lang_path . $user_lang . '/acp_common.php';
        if ($this->filesystem->exists($primary_lang_file))
        {
            $this->language->add_lang('acp_common', 'mundophpbb/custom404');
            error_log('.: Loaded language file: ' . $primary_lang_file . ' at ' . date('Y-m-d H:i:s'));
        }
        else
        {
            // Fallback to English if the primary language file is missing
            $fallback_lang_file = $lang_path . $fallback_lang . '/acp_common.php';
            if ($this->filesystem->exists($fallback_lang_file))
            {
                $this->language->add_lang('acp_common', 'mundophpbb/custom404', $fallback_lang);
                error_log('.: Fallback to English language file: ' . $fallback_lang_file . ' at ' . date('Y-m-d H:i:s'));
            }
            else
            {
                error_log('.: No language file found for ' . $user_lang . ' or fallback ' . $fallback_lang . ' at ' . date('Y-m-d H:i:s'));
            }
        }

        // Load additional core language files if needed
        $this->language->add_lang('common'); // For general phpBB translations
    }
}