<?php
/**
 * Upload Handler Service for Custom 404 Page Extension
 *
 * @package mundophpbb\custom404\service
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 */
namespace mundophpbb\custom404\service;

class upload_handler
{
    /** @var \phpbb\filesystem\filesystem */
    protected $filesystem;

    /** @var \phpbb\language\language */
    protected $language;

    /** @var \phpbb\config\config */
    protected $config;

    /**
     * Constructor
     *
     * @param \phpbb\filesystem\filesystem $filesystem
     * @param \phpbb\language\language $language
     * @param \phpbb\config\config $config
     */
    public function __construct(
        \phpbb\filesystem\filesystem $filesystem,
        \phpbb\language\language $language,
        \phpbb\config\config $config
    ) {
        $this->filesystem = $filesystem;
        $this->language = $language;
        $this->config = $config;
    }

    /**
     * Handle file upload
     *
     * @param array $file The uploaded file data from $request->file()
     * @param string $upload_dir The directory to save the uploaded file
     * @param array $allowed_extensions Array of allowed file extensions
     * @param int $max_file_size Maximum file size in bytes
     * @return string|null The path to the uploaded file or null if no file was uploaded
     * @throws \Exception If upload fails or validation errors occur
     */
    public function handle_upload(array $file, string $upload_dir, array $allowed_extensions, int $max_file_size): ?string
    {
        // Check if a file was uploaded
        if (empty($file['name'])) {
            return null;
        }

        // Validate file extension
        $file_ext = strtolower(pathinfo($file['name'], PATHINFO_EXTENSION));
        if (!in_array($file_ext, $allowed_extensions)) {
            throw new \Exception($this->language->lang('ACP_CUSTOM404_INVALID_FILE_TYPE'));
        }

        // Validate file size
        if ($file['size'] > $max_file_size) {
            throw new \Exception($this->language->lang('ACP_CUSTOM404_FILE_TOO_LARGE'));
        }

        // Generate unique filename
        $new_filename = uniqid() . '.' . $file_ext;
        $destination = rtrim($upload_dir, '/') . '/' . $new_filename;

// Ensure upload directory exists
if (!$this->filesystem->exists($upload_dir)) {
    $this->filesystem->mkdir($upload_dir, 0777);
}

// Get the original filename and sanitize it
$new_filename = basename($file['name']); // Usa o nome original do arquivo, sanitizado
$destination = $upload_dir . '/' . $new_filename;

// Check if file already exists to avoid overwriting
$counter = 1;
$info = pathinfo($new_filename);
$base_name = $info['filename'];
$extension = isset($info['extension']) ? '.' . $info['extension'] : '';

while ($this->filesystem->exists($destination)) {
    $new_filename = $base_name . '_' . $counter . $extension;
    $destination = $upload_dir . '/' . $new_filename;
    $counter++;
}

// Move uploaded file
if (!move_uploaded_file($file['tmp_name'], $destination)) {
    throw new \Exception($this->language->lang('ACP_CUSTOM404_UPLOAD_FAILED'));
}

// Return the relative path for storage in config
return 'ext/mundophpbb/custom404/styles/all/theme/images/' . $new_filename;
    }
}